@@ -10,7 +10,7 @@
 #   Andy Wardley   <abw@wardley.org>
 #
 # COPYRIGHT
-#   Copyright (C) 2000-2007 Andy Wardley.  All Rights Reserved.
+#   Copyright (C) 2000-2022 Andy Wardley.  All Rights Reserved.
 #
 #   This module is free software; you can redistribute it and/or
 #   modify it under the same terms as Perl itself.
@@ -48,11 +48,12 @@ sub new {
     # the data it contains
     if (blessed($data) && $data->isa('Template::Iterator')) {
         ($data, $error) = $data->get_all();
-        return $class->error("iterator failed to provide data for table: ",
-                             $error)
-            if $error;
+        return $class->error(
+            "iterator failed to provide data for table: ",
+            $error
+        ) if $error;
     }
-        
+
     return $class->error('invalid table data, expecting a list')
         unless ref $data eq 'ARRAY';
 
@@ -75,7 +76,7 @@ sub new {
         }
         else {
             $coloff = $rows - $overlap;
-            $cols = int ($size / $coloff) 
+            $cols = int ($size / $coloff)
                 + ($size % $coloff > $overlap ? 1 : 0)
             }
     }
@@ -87,7 +88,7 @@ sub new {
             $coloff = 1;
         }
         else {
-            $coloff = int ($size / $cols) 
+            $coloff = int ($size / $cols)
                 + ($size % $cols > $overlap ? 1 : 0);
             $rows = $coloff + $overlap;
         }
@@ -97,7 +98,7 @@ sub new {
         $cols = 1;
         $coloff = 0;
     }
-    
+
     bless {
         _DATA    => $data,
         _SIZE    => $size,
@@ -113,14 +114,14 @@ sub new {
 #------------------------------------------------------------------------
 # row($n)
 #
-# Returns a reference to a list containing the items in the row whose 
+# Returns a reference to a list containing the items in the row whose
 # number is specified by parameter.  If the row number is undefined,
 # it calls rows() to return a list of all rows.
 #------------------------------------------------------------------------
 
 sub row {
     my ($self, $row) = @_;
-    my ($data, $cols, $offset, $size, $pad) 
+    my ($data, $cols, $offset, $size, $pad)
         = @$self{ qw( _DATA _NCOLS _COLOFF _SIZE _PAD) };
     my @set;
 
@@ -129,13 +130,16 @@ sub row {
         unless defined $row;
 
     return () if $row >= $self->{ _NROWS } || $row < 0;
-    
+
     my $index = $row;
 
     for (my $c = 0; $c < $cols; $c++) {
-        push(@set, $index < $size 
-             ? $data->[$index] 
-             : ($pad ? undef : ()));
+        push(
+            @set,
+            $index < $size
+                ? $data->[$index]
+                : ($pad ? undef : ())
+        );
         $index += $offset;
     }
     return \@set;
@@ -170,8 +174,12 @@ sub col {
         $end = $size - 1;
     }
     return () if $start >= $size;
-    return [ @$data[$start..$end], 
-             $self->{ _PAD } ? ((undef) x $blanks) : () ];
+    return [
+        @$data[$start..$end],
+        $self->{ _PAD }
+            ? ((undef) x $blanks)
+            : ()
+    ];
 }
 
 
@@ -232,30 +240,30 @@ Template::Plugin::Table - Plugin to present data in a table
 =head1 SYNOPSIS
 
     [% USE table(list, rows=n, cols=n, overlap=n, pad=0) %]
-    
+
     [% FOREACH item IN table.row(n) %]
        [% item %]
     [% END %]
-    
+
     [% FOREACH item IN table.col(n) %]
        [% item %]
     [% END %]
-    
+
     [% FOREACH row IN table.rows %]
        [% FOREACH item IN row %]
           [% item %]
        [% END %]
     [% END %]
-    
+
     [% FOREACH col IN table.cols %]
        [% col.first %] - [% col.last %] ([% col.size %] entries)
     [% END %]
 
 =head1 DESCRIPTION
 
-The C<Table> plugin allows you to format a list of data items into a 
+The C<Table> plugin allows you to format a list of data items into a
 virtual table.  When you create a C<Table> plugin via the C<USE> directive,
-simply pass a list reference as the first parameter and then specify 
+simply pass a list reference as the first parameter and then specify
 a fixed number of rows or columns.
 
     [% USE Table(list, rows=5) %]
@@ -270,14 +278,14 @@ name for the plugin as per regular Template Toolkit syntax.
 The plugin then presents a table based view on the data set.  The data
 isn't actually reorganised in any way but is available via the C<row()>,
 C<col()>, C<rows()> and C<cols()> as if formatted into a simple two dimensional
-table of C<n> rows x C<n> columns.  
+table of C<n> rows x C<n> columns.
 
-So if we had a sample C<alphabet> list contained the letters 'C<a>' to 'C<z>', 
-the above C<USE> directives would create plugins that represented the following 
+So if we had a sample C<alphabet> list contained the letters 'C<a>' to 'C<z>',
+the above C<USE> directives would create plugins that represented the following
 views of the alphabet.
 
     [% USE table(alphabet, ... %]
-    
+
     rows=5                  cols=5
     a  f  k  p  u  z        a  g  m  s  y
     b  g  l  q  v           b  h  n  t  z
@@ -293,7 +301,7 @@ methods.
     [% FOREACH item = table.row(0) %]
        # [% item %] set to each of [ a f k p u z ] in turn
     [% END %]
-    
+
     [% FOREACH item = table.col(2) %]
        # [% item %] set to each of [ m n o p q r ] in turn
     [% END %]
@@ -301,7 +309,7 @@ methods.
 Data in rows is returned from left to right, columns from top to
 bottom.  The first row/column is 0.  By default, rows or columns that
 contain empty values will be padded with the undefined value to fill
-it to the same size as all other rows or columns.  
+it to the same size as all other rows or columns.
 
 For example, the last row (row 4) in the first example would contain the
 values C<[ e j o t y undef ]>. The Template Toolkit will safely accept these
@@ -314,7 +322,7 @@ to test if the value is set.
       [% END %]
    [% END %]
 
-You can explicitly disable the C<pad> option when creating the plugin to 
+You can explicitly disable the C<pad> option when creating the plugin to
 returned shortened rows/columns where the data is empty.
 
    [% USE table(alphabet, cols=5, pad=0) %]
@@ -352,7 +360,7 @@ This produces the following output:
     [ j - r (9 letters) ]
     [ s - z (8 letters) ]
 
-We can also use the general purpose C<join> virtual method which joins 
+We can also use the general purpose C<join> virtual method which joins
 the items of the list using the connecting string specified.
 
     [% USE table(alphabet, cols=5) %]
@@ -403,13 +411,13 @@ first called on the iterator to return all remaining items. These are then
 available via the usual Table interface.
 
     [% USE DBI(dsn,user,pass) -%]
-    
+
     # query() returns an iterator
     [% results = DBI.query('SELECT * FROM alphabet ORDER BY letter') %]
 
     # pass into Table plugin
     [% USE table(results, rows=8 overlap=1 pad=0) -%]
-    
+
     [% FOREACH row = table.cols -%]
        [% row.first.letter %] - [% row.last.letter %]:
           [% row.join(', ') %]
@@ -421,7 +429,7 @@ Andy Wardley E<lt>abw@wardley.orgE<gt> L<http://wardley.org/>
 
 =head1 COPYRIGHT
 
-Copyright (C) 1996-2007 Andy Wardley.  All Rights Reserved.
+Copyright (C) 1996-2022 Andy Wardley.  All Rights Reserved.
 
 This module is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
